﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using WindowsInput;
using Hardcodet.Wpf.TaskbarNotification;
using System.IO.Ports;
using System.Text.RegularExpressions;

namespace Volumecontroller
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        SerialPort MySerialPort = new SerialPort();  //Define serial connection
        private TaskbarIcon tb;   //Define taskbar icon

        public MainWindow()
        {
            InitializeComponent();
           Hide();

            message_box.FontSize = 12;
            COM_text_box.Text = Volumecontroller.Properties.Settings.Default.default_port_num;

            MySerialPort.BaudRate = 9600;
            MySerialPort.PortName = Properties.Settings.Default.default_port; //Set the default port
            connect(MySerialPort);
            check_state();

            MySerialPort.DataReceived += new SerialDataReceivedEventHandler(DataReceivedHandler); //Receive data from controller

        }


        public void connect(SerialPort Comport)  //Connect to com
        {
            try
            {
                Comport.Open();
                string print = String.Format("Successfully connected to: {0}", Comport.PortName);
                message_box.Text = print;

            }
            catch
            {
                string print2 = String.Format("Couldn't open: {0}", Comport.PortName);
                message_box.Text = print2;
                string printout = String.Format("               Error! \n\n{0}", print2);
                MessageBox.Show(printout, "Volume Controller");
                Comport.Close();
            }

        }

        public void check_state()  // Check if the port is open
        {
            if (MySerialPort.IsOpen)
            {

                infotext.Text = "The port is open";
            }
            else
            {

                infotext.Text = "The port is closed";
            }
        }

        public void DataReceivedHandler(object sender, SerialDataReceivedEventArgs e)  // React to received data
        {
            SerialPort myport = (SerialPort)sender;
            int data = myport.ReadByte();

            switch (data)
            {
                case 49:
                    InputSimulator.SimulateKeyPress(VirtualKeyCode.VOLUME_UP);

                    this.Dispatcher.Invoke((Action)(() =>  //Update GUI label
                    {
                        command_text.Text = "Volume up";
                    }));

                    break;
                case 50:
                    InputSimulator.SimulateKeyPress(VirtualKeyCode.VOLUME_DOWN);

                    this.Dispatcher.Invoke((Action)(() =>  //Update GUI label
                    {
                        command_text.Text = "Volume down";
                    }));

                    break;

                case 51:
                    InputSimulator.SimulateKeyPress(VirtualKeyCode.MEDIA_PLAY_PAUSE);


                    this.Dispatcher.Invoke((Action)(() =>  //Update GUI label
                    {
                        command_text.Text = "Play / Pause";
                    }));

                    break;
                case 52:
                    InputSimulator.SimulateKeyPress(VirtualKeyCode.MEDIA_NEXT_TRACK);


                    this.Dispatcher.Invoke((Action)(() =>  //Update GUI label
                    {
                        command_text.Text = "Next track";
                    }));

                    break;
                case 53:
                    InputSimulator.SimulateKeyPress(VirtualKeyCode.MEDIA_PREV_TRACK);


                    this.Dispatcher.Invoke((Action)(() =>  //Update GUI label
                    {
                        command_text.Text = "Previous track";
                    }));

                    break;
                case 54:
                    InputSimulator.SimulateKeyPress(VirtualKeyCode.MEDIA_STOP);


                    this.Dispatcher.Invoke((Action)(() =>  //Update GUI label
                    {
                        command_text.Text = "Stop";
                    }));

                    break;
            }


        }

        private void button2_Click(object sender, RoutedEventArgs e)  //Mute button
        {
            InputSimulator.SimulateKeyPress(VirtualKeyCode.VOLUME_MUTE);
        }


        private void button3_Click(object sender, RoutedEventArgs e)  //Set default COM port button
        {
            if (Regex.IsMatch(COM_text_box.Text, @"^\d+$"))
            {
                try
                {
                    string com = this.COM_text_box.Text;
                    Volumecontroller.Properties.Settings.Default.default_port_num = com;
                    string comport = String.Format("COM{0}", this.COM_text_box.Text);
                    string output = String.Format("Default port set to: {0}", comport);
                    Volumecontroller.Properties.Settings.Default.default_port = comport;
                    Volumecontroller.Properties.Settings.Default.Save();
                    message_box.Text = output;

                    MySerialPort.PortName = Volumecontroller.Properties.Settings.Default.default_port;  // Change the port name
                }

                catch
                {
                    message_box.Text = "Couldn't set the default COM port!";
                }
            }
            else
            {
                message_box.Text = "Couldn't set the default COM port!\nThe input must be a whole number!";
            }
        }


        private void button_Click(object sender, RoutedEventArgs e)   //Connect
        {
            connect(MySerialPort);
            check_state();
        }

        private void button2_Click_1(object sender, RoutedEventArgs e)  // Disconnect
        {
            try
            {
                MySerialPort.Close();
                message_box.Text = "Port closed!";
            }
            catch
            {
                message_box.Text = "Failed to close port!";
            }
            check_state();
        }

        private void button1_Click(object sender, RoutedEventArgs e)   // Pause button
        {
            InputSimulator.SimulateKeyPress(VirtualKeyCode.MEDIA_PLAY_PAUSE);
        }

        private void button4_Click(object sender, RoutedEventArgs e)
        {
            Hide();
        }

        private void back_button_Click(object sender, RoutedEventArgs e)
        {
            InputSimulator.SimulateKeyPress(VirtualKeyCode.MEDIA_PREV_TRACK);
        }

        private void for_button_Click(object sender, RoutedEventArgs e)
        {
            InputSimulator.SimulateKeyPress(VirtualKeyCode.MEDIA_NEXT_TRACK);

        }
    }

    public class ShowMessageCommand : ICommand  // Taskbar functions
    {
        public void Execute(object parameter)
        {
            ((MainWindow)System.Windows.Application.Current.MainWindow).Show();  //Brings upp the window 
        }

        public bool CanExecute(object parameter)
        {
            return true;
        }

        public event EventHandler CanExecuteChanged;
    }

}
